@@ -34,8 +34,8 @@
 #include "daisyDrawers.cpp"
 #include "daisyPlug_trash.cpp"
 #include "daisyIcon.cpp"
-
-
+#include "daisyMediaControl.cpp"
+#include "daisyDialogs.cpp"
 
 
 
@@ -67,12 +67,11 @@ Plasma::Containment(parent, args), m_backframe(this), m_backframe_t(this), m_hov
   
   QList< Plasma::IconWidget* > m_widgets;
   QList< Plasma::IconWidget* > m_drawericons;
+  QList< Plasma::IconWidget* > m_taskicons;
   QList< Plasma::FrameSvg* > m_indicons;
   
   
-  
-  
-  
+  Plasma::Dialog m_drawdial;
   
   
   m_trashfull = false;
@@ -90,17 +89,29 @@ Plasma::Containment(parent, args), m_backframe(this), m_backframe_t(this), m_hov
   drawerpopspot_y = 0;
   m_activewindow = int(-1);
   m_trash_dir = QString( QDir("").homePath() + "/.local/share/Trash/files/");
-  m_icodimension = int(50);
-  m_icospacing = 0.350;
+  m_icodimension = int(52);
+  m_icospacing = 0.250;
   m_position = QString("buttom_center");
   m_orientation = QString("horizontal");
   m_backopacity = 1.0;
   m_hidingdelay = 3000;
+  m_workwid = 0;
+  m_toupwid = 0;
+  m_focusedWindow = 0;
   
   m_drawersize = 38;
   m_drawerrotation = 1.50;
   
   
+  last_icopos << -1000 << -1000;
+  total_taskpreviews = 0;
+  m_max_taskpreviews = 20;//FIXME
+  m_taskpreviews = true;
+  m_showtaskoflauncher = true;
+  
+  m_showclosetask = false;
+  m_middleClickAction = QString("shade");
+  
   
   
   m_trash_spot = int(3);//FIXME
@@ -108,6 +119,7 @@ Plasma::Containment(parent, args), m_backframe(this), m_backframe_t(this), m_hov
   m_total_plugs = int(1);
   m_show_trash = true;
   m_circpos = QPointF(62,268);
+  m_clickpos = QPointF(0,0);
   
   m_hoverpz = QPointF( -100000, -100000 );
   m_hoversz = QSizeF(m_icodimension,m_icodimension);
@@ -137,15 +149,22 @@ Plasma::Containment(parent, args), m_backframe(this), m_backframe_t(this), m_hov
   m_rz_tmr->setInterval(700);
   connect( m_rz_tmr, SIGNAL(timeout()), this, SLOT(delayResize()) );
   
-  m_rp_tmr = new QTimer();
-  m_rp_tmr->setInterval(700);
-  connect( m_rp_tmr, SIGNAL(timeout()), this, SLOT(delay_Resize()) );
-  
-  
   m_tracker_tmr = new QTimer();
   m_tracker_tmr->setInterval(m_hidingdelay);
   connect( m_tracker_tmr, SIGNAL(timeout()), this, SLOT(mouseTracker()) );
   
+  m_dlx_tmr = new QTimer();
+  m_dlx_tmr->setInterval(50);
+  connect( m_dlx_tmr, SIGNAL(timeout()), this, SLOT(delayExec()) );
+  
+  m_dlt_tmr = new QTimer();
+  m_dlt_tmr->setInterval(50);
+  connect( m_dlt_tmr, SIGNAL(timeout()), this, SLOT(delayShade()) );
+  
+  m_uti_tmr = new QTimer();
+  m_uti_tmr->setInterval(10);
+  connect( m_uti_tmr, SIGNAL(timeout()), this, SLOT(delayTaskIcoChange()) );
+  
   
   
   // watchers
@@ -187,6 +206,9 @@ Plasma::Containment(parent, args), m_backframe(this), m_backframe_t(this), m_hov
   m_act_newinstance = new QAction(KIcon("application-x-executable"), i18n("New instance"), this);
   connect(m_act_newinstance, SIGNAL(triggered()), this, SLOT(launNewInstance()));
   
+  m_act_closetask = new QAction(KIcon("window-close"), i18n("Close"), this);
+  connect(m_act_closetask, SIGNAL(triggered()), this, SLOT(taskClosing()));
+  
   
   
   QList <QAction*> m_acta;
@@ -208,6 +230,7 @@ Plasma::Containment(parent, args), m_backframe(this), m_backframe_t(this), m_hov
   setBackgroundHints(NoBackground);
   setAspectRatioMode(Plasma::KeepAspectRatio);
   setContentsMargins(0, 0, 0, 0);
+  setWindowFrameMargins(0,0,0,0);
   setFocusPolicy( Qt::NoFocus );
   resize(330,330);
   setPos(m_circpos);
@@ -226,7 +249,7 @@ void daisy::init()
 {
   
   if( location() == 3 ){m_inpanel = true;setMinimumSize(8,8);}
-  else{m_inpanel = false;setMinimumSize(170,170);}
+  else{m_inpanel = false;setMinimumSize(100,100);}
   
   // tasks
   m_groupManager = new TaskManager::GroupManager(this);
@@ -240,7 +263,6 @@ void daisy::init()
   connect( KWindowSystem::self(), SIGNAL( windowChanged(WId, const unsigned long* ) ), this, SLOT( windowChanged(WId, const unsigned long* ) ) );
   
   
-  
   readConfiguration();
   getLauncherValues();
   emit spaceReserver(true);
@@ -252,9 +274,6 @@ void daisy::init()
   m_paint_tmr->start();
   raise();
   
-  
-  
-  
 }
 
 
@@ -283,7 +302,6 @@ void daisy::paintDesktop(QPainter *p, const QRect &contentsRect)
   QList<double> m_ico_szX = get_val[2];QList<double> m_ico_szY = get_val[3];
   QList<double> m_back_sz = get_val[4];
   QList<double> m_indico_posX = get_val[5];QList<double> m_indico_posY = get_val[6];
-  
   p->setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform | QPainter::TextAntialiasing);
   
   
@@ -304,9 +322,79 @@ void daisy::paintDesktop(QPainter *p, const QRect &contentsRect)
   
   
   
-  // We place the launchers
   if ( m_type == QString("circular_dock") ||  m_type == QString("standard_dock") )
   {
+    
+    
+    //Thumbnails
+    if ( m_taskpreviews && m_type == QString("standard_dock") )
+    {
+      for (int i = 0; i < total_taskpreviews; ++i)
+      {
+	
+	if ( m_orientation== QString("horizontal") )
+	{
+	  m_taskicons[i]->setGeometry( QRect( last_icopos[0] + ( ( i* (m_icodimension+(m_icodimension*m_icospacing)) ) ), last_icopos[1], m_icodimension, m_icodimension) );
+	}
+	else
+	{
+	  m_taskicons[i]->setGeometry( QRect( last_icopos[0] , last_icopos[1] + ( ( i* (m_icodimension+(m_icodimension*m_icospacing)) ) ), m_icodimension, m_icodimension) );
+	}
+	
+	
+	if ( m_showmirror )
+	{
+	  p->setOpacity(0.55);
+	  QRect mirrect;QImage image_tmp;
+	  QPixmap pixmap_tmp = m_taskicons[i]->icon().pixmap(m_icodimension,m_icodimension);
+	  
+	  if ( m_position == QString("buttom_center") || m_position == QString("buttom_left") || m_position == QString("buttom_right") )
+	  {
+	    pixmap_tmp = pixmap_tmp.copy(0, (pixmap_tmp.height() * 0.66), pixmap_tmp.width(), (pixmap_tmp.height() * 0.33));
+	    image_tmp = pixmap_tmp.toImage();
+	    image_tmp = image_tmp.mirrored(true, true);image_tmp = image_tmp.mirrored(true, false);
+	    mirrect = QRect( last_icopos[0] + ( ( i* (m_icodimension+(m_icodimension*m_icospacing)) ) ),
+			      last_icopos[1]+(m_icodimension-(m_icodimension/15)),
+			      m_icodimension,
+			      (m_icodimension)/3 );
+	  }
+	  else if ( m_position == QString("top_center") || m_position == QString("top_left") || m_position == QString("top_right") )
+	  {
+	    pixmap_tmp = pixmap_tmp.copy(0, 0, pixmap_tmp.width(), (pixmap_tmp.height() * 0.33));
+	    image_tmp = pixmap_tmp.toImage();
+	    image_tmp = image_tmp.mirrored(true, true);image_tmp = image_tmp.mirrored(true, false);
+	    mirrect = QRect( last_icopos[0] + ( ( i* (m_icodimension+(m_icodimension*m_icospacing)) ) ),
+			     last_icopos[1]-((m_icodimension-(m_icodimension/15))/3),
+			     m_icodimension, (m_icodimension)/3 );
+	  }
+	  else if ( m_position == QString("left_center") || m_position == QString("left_top") || m_position == QString("left_buttom") )
+	  {
+	    pixmap_tmp = pixmap_tmp.copy( 0, 0, (pixmap_tmp.width() * 0.33), pixmap_tmp.height() );
+	    image_tmp = pixmap_tmp.toImage();
+	    image_tmp = image_tmp.mirrored(true, false);
+	    mirrect = QRect( last_icopos[0]-(m_icodimension-(m_icodimension/15))/3,
+			     last_icopos[1] + ( ( i* (m_icodimension+(m_icodimension*m_icospacing)) ) ),
+			     (m_icodimension)/3, m_icodimension );
+	  }
+	  else if ( m_position == QString("right_center") || m_position == QString("right_top") || m_position == QString("right_buttom") )
+	  {
+	    pixmap_tmp = pixmap_tmp.copy( (pixmap_tmp.width() * 0.66), 0, (pixmap_tmp.width() * 0.33), pixmap_tmp.height() );
+	    image_tmp = pixmap_tmp.toImage();
+	    image_tmp = image_tmp.mirrored(true, false);
+	    mirrect = QRect( last_icopos[0]+(m_icodimension-(m_icodimension/15)),
+			     last_icopos[1] + ( ( i* (m_icodimension+(m_icodimension*m_icospacing)) ) ),
+			     (m_icodimension)/3, m_icodimension );
+	  }
+	  
+	  p->drawImage ( mirrect, image_tmp );
+	  p->setOpacity(1.0);
+	}
+      }
+    }
+    
+    
+    
+    //Launchers
     for (int i = 0; i < m_widgets.size(); ++i)
     {
       m_widgets[m_domino[i]]->setGeometry( QRect( m_ico_posX[i], m_ico_posY[i], m_ico_szX[i]+m_zoomszdiff[m_domino[i]], m_ico_szY[i]+m_zoomszdiff[m_domino[i]]) );
@@ -359,13 +447,14 @@ void daisy::paintDesktop(QPainter *p, const QRect &contentsRect)
 	p->setOpacity(1.0);
       }
       
-      
+      //Indicons
       if (m_showindicons)
       {
 	QString indiconpath_tmp = QString("inactive");
 	if ( m_domino[i] == m_startingwindow )indiconpath_tmp = QString("starting");
 	else if ( m_domino[i] == m_closingwindow )indiconpath_tmp = QString("closing");
 	else if ( m_domino[i] == m_activewindow )indiconpath_tmp = QString("focused");
+	else if ( m_attentionwindow.contains(m_domino[i]) )indiconpath_tmp = QString("attention");
 	else if ( m_activetasks.contains(m_domino[i]) )indiconpath_tmp = QString("active");
 	m_indicons[m_domino[i]]->setElementPrefix(indiconpath_tmp);
 	m_indicons[m_domino[i]]->resizeFrame( QSizeF( m_ico_szX[i]+m_zoomszdiff[m_domino[i]], m_ico_szY[i]+m_zoomszdiff[m_domino[i]] ) );
@@ -382,11 +471,10 @@ void daisy::paintDesktop(QPainter *p, const QRect &contentsRect)
 	  p->setOpacity(1.0);
 	}
       }
-      
     }
     
     
-    
+    //Drawers
     for (int i = 0; i < m_drawericons.size(); ++i)
     {
       QRect drawerrect_tmp;
@@ -411,10 +499,6 @@ void daisy::paintDesktop(QPainter *p, const QRect &contentsRect)
       m_drawericons[i]->setGeometry(drawerrect_tmp);
     }
     
-    
-    
-    
-    
   }
   
   
@@ -473,7 +557,7 @@ void daisy::constraintsEvent(Plasma::Constraints constraints)
   else
   {
     m_inpanel = false;
-    if ( m_type!=QString("standard_dock") )setMinimumSize(170,170);
+    if ( m_type!=QString("standard_dock") )setMinimumSize(100,100);
   }
   
   if (constraints & Plasma::FormFactorConstraint)
@@ -509,6 +593,10 @@ void daisy::wheelEvent(QGraphicsSceneWheelEvent *event)
     else {myProcess->start(getPlayerComs()[9]);}
   }
   
+  else if ( m_type == QString("standard_dock") )
+  {scrollTasks(delta);}
+  
+  
   event->accept();
 }
 
@@ -549,39 +637,54 @@ void daisy::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
 
 
 //-----------------------------------------------------------------------------
-// Mouse press event
+// Mouse press event FIXME
 void daisy::mousePressEvent(QGraphicsSceneMouseEvent *event)
 {
   startPos = event->scenePos();
-  QPointF clickpos = event->pos();
+  m_clickpos = event->pos();
   setFlag(QGraphicsItem::ItemIsMovable, true);
   m_dexval = -1;
+  m_dlx_tmr->stop();m_dlt_tmr->stop();
   
-  if ( contentsRect().contains(clickpos) )
+  if ( contentsRect().contains(m_clickpos) )
   {
     if (event->buttons () == Qt::LeftButton )
     {}
+    
     else if (event->buttons () == Qt::MidButton )
     {
       if ( m_type == QString("media_controller") )
       {QProcess *myProcess = new QProcess(this);myProcess->start(getPlayerComs()[7]);}
+      
       else
       {
-	m_dexval = getContextLauncher(clickpos);
-	if ( m_dexval != -1 && m_dexval != m_trash_spot )
+	m_dexval = getContextLauncher(m_clickpos);
+	if ( m_dexval != m_trash_spot )
 	{
-	  if ( m_activetasks.contains(m_dexval) )
+	  if ( m_dexval !=-1 )
 	  {
-	    WId widlaunch_tmp = matchTask( m_alias[m_dexval] );
-	    if ( widlaunch_tmp != 0 )
-	    {toggShadeTask( widlaunch_tmp );}
+	    if (m_middleClickAction == QString("shade"))
+	    {
+	      m_workwid = matchTask( m_alias[m_dexval] );
+	      if ( m_activetasks.contains(m_dexval) && m_workwid != 0 )
+	      {m_dlt_tmr->start();}
+	    }
+	    else if (m_middleClickAction == QString("run"))
+	    {m_dlx_tmr->start();}
+	  }
+	  else if ( m_taskpreviews && m_type == QString("standard_dock") )
+	  {
+	    m_dexval = getContextTask(m_clickpos);
+	    if ( m_dexval !=-1 )
+	    {
+	      m_workwid = showedTasks[m_dexval];
+	      m_dlt_tmr->start();
+	    }
 	  }
 	}
       }
     }
-    
   }
-  
   Containment::mousePressEvent(event);
 }
 
@@ -623,7 +726,7 @@ void daisy::dragLeaveEvent(QGraphicsSceneDragDropEvent *event)
 // Mouse move event
 void daisy::mouseMoveEvent (QGraphicsSceneMouseEvent *event)
 {
-  QPointF clickpos = event->pos();
+  m_clickpos = event->pos();
   
   if(event->buttons() & Qt::LeftButton)
   {
@@ -651,6 +754,8 @@ void daisy::startDrag(QGraphicsSceneMouseEvent *event)
   QStringList list_tmp;
   list_tmp = m_values[m_dexval][0].split(QRegExp("\\s+"));
   KService::Ptr service = KService::serviceByStorageId( list_tmp[0] );
+  
+  
   if (service && service->isValid())
   {
     QString path = service->entryPath();
@@ -694,7 +799,7 @@ void daisy::startDrag(QGraphicsSceneMouseEvent *event)
 // Drop event
 void daisy::dropEvent(QGraphicsSceneDragDropEvent* event)
 {
-  QPointF clickpos = event->pos();
+  m_clickpos = event->pos();
   
   if (!KUrl::List::canDecode(event->mimeData()))
   {event->ignore();return;}
@@ -705,17 +810,17 @@ void daisy::dropEvent(QGraphicsSceneDragDropEvent* event)
     
     if ( payload != m_url_drag )
     {
-      m_dexval = getContextLauncher(clickpos);
+      m_dexval = getContextLauncher(m_clickpos);
       if ( m_dexval != -1 )
       {if (m_dexval==m_trash_spot && m_show_trash)moveToTrash(payload);}
-      else{emit addLauncher(clickpos, payload);}
+      else{emit addLauncher(m_clickpos, payload);}
       event->accept();
     }
     
     else
     {
       if ( m_type == QString("standard_dock") )
-      {emit moveLauncher( clickpos );}
+      {emit moveLauncher( m_clickpos );}
     }
   }
   
@@ -743,7 +848,7 @@ void daisy::screenChanged(int wasScreen, int isScreen, Plasma::Containment *cont
 // Context menu event
 void daisy::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
 {
-  QPointF clickpos = event->pos();
+  m_clickpos = event->pos();
   
   if (m_dialogLauncher == 0)
   {
@@ -751,9 +856,27 @@ void daisy::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
     {Applet::contextMenuEvent(event);}
     else
     {
-      m_dexval = getContextLauncher(clickpos);
+      m_dexval = getContextLauncher(m_clickpos);
       if ( m_dexval == -1 )
-      {Applet::contextMenuEvent(event);}
+      {
+	if ( m_taskpreviews && m_type == QString("standard_dock") )
+	  m_dexval = getContextTask(m_clickpos);
+	if ( m_dexval == -1 )
+	  Applet::contextMenuEvent(event);
+	else
+	{
+	  if (!m_drawerscleared) clearDrawers();
+	  m_task = TaskManager::TaskManager::self()->findTask( showedTasks[m_dexval] );
+	  if (m_task)
+	  {
+	    TaskManager::BasicMenu *taskMenu;TaskManager::TaskItem *taskitem;
+	    taskitem = new TaskManager::TaskItem::TaskItem(this, m_task);
+	    taskMenu = new TaskManager::BasicMenu( qobject_cast<QWidget*>(this), taskitem, m_groupManager );
+	    taskMenu->popup( event->screenPos() );
+	  }
+	}
+      }
+      
       else
       {
 	if (!m_drawerscleared) clearDrawers();
@@ -924,14 +1047,31 @@ QPainterPath daisy::shape() const
 // Get the right-clicked launcher
 int daisy::getContextLauncher( QPointF &p_click )
 {
+  m_dexval = -1;
   for (int i = 0; i < m_widgets.size(); ++i)
-  {if (m_widgets[i]->geometry().contains(p_click))return i;}
+  {
+    if ( m_widgets[i]->geometry().contains(p_click) )
+      m_dexval = i;
+  }
+  return m_dexval;
+}
+
+
+//-----------------------------------------------------------------------------
+// Get the right-clicked task
+int daisy::getContextTask( QPointF &p_click )
+{
+  for (int i = 0; i < total_taskpreviews; ++i)
+  {if (m_taskicons[i]->geometry().contains(p_click))return i;}
   return -1;
 }
 
 
 
 
+
+
+
 //-----------------------------------------------------------------------------
 // Create the Menus
 void daisy::createMenus()
@@ -1040,24 +1180,28 @@ void daisy::resizeRepos( bool m_resetpos )
 {
   if (!m_drawerscleared) clearDrawers();
   m_tracker_tmr->setInterval(m_hidingdelay);m_tracker_tmr->stop();
-  if ( !isVisible() ){setVisible(true);}
+  
   
   if ( m_type == QString("circular_dock") || m_type == QString("media_controller")  )
   {
     m_domino.clear();m_domino << 1 << 6 << 8 << 3 << 5 << 2 << 4 << 7 << 0;
-    
     setImmutability(Plasma::Mutable);
     setAspectRatioMode(Plasma::KeepAspectRatio);
     setContentsMargins(0, 0, 0, 0);
-    if (!m_inpanel)setMinimumSize(170,170);
     if (m_resetpos)
     {
-      resize(330,330);setPos(m_circpos);
+      // Plasma Resizing Bug (KDE 4.3).
+      setMinimumSize(330,330);
+      setMaximumSize(330,330);
+      resize(330,330);
+      setPos(QPointF(62,268));
+      QTimer::singleShot(150, this, SLOT(resetResize()));
     }
-    
+    if (!m_inpanel)setMinimumSize(100,100);
   }
   
   
+  
   else if ( m_type == QString("standard_dock") )
   {
     
@@ -1081,13 +1225,12 @@ void daisy::resizeRepos( bool m_resetpos )
     setImmutability(Plasma::Mutable);
     setAspectRatioMode(Plasma::IgnoreAspectRatio);
     setContentsMargins(0, 0, 0, 0);
-    setMinimumSize(10,10);
     
     
     //Horz
     if ( m_position == QString("buttom_center") )
     {
-      pos_x = ( rect.width()/2)-( (  (((m_icodimension/1.7)*2)-(m_icodimension*m_icospacing))  + (m_total_launchers*  (m_icodimension+(m_icodimension*m_icospacing))  ))/2  ); pos_y = rect.height()- (m_icodimension+((m_icodimension/5)*2) );
+      pos_x = ( rect.width()/2)-( (  (((m_icodimension/1.7)*2)-(m_icodimension*m_icospacing))  + (m_total_launchers*  (m_icodimension+(m_icodimension*m_icospacing))  )+((total_taskpreviews*(m_icodimension+(m_icodimension*m_icospacing))) + (m_icospacing)))/2  ); pos_y = rect.height()- (m_icodimension+((m_icodimension/5)*2) );
       m_orientation = QString("horizontal");
     }
     else if ( m_position == QString("buttom_left") )
@@ -1097,44 +1240,41 @@ void daisy::resizeRepos( bool m_resetpos )
     }
     else if ( m_position == QString("buttom_right") )
     {
-      pos_x = ( rect.width())-( (  (((m_icodimension/1.7)*2)-(m_icodimension*m_icospacing))  + (m_total_launchers*  (m_icodimension+(m_icodimension*m_icospacing))  ))  ); pos_y = rect.height()- (m_icodimension+((m_icodimension/5)*2) );
+      pos_x = ( rect.width())-( (  (((m_icodimension/1.7)*2)-(m_icodimension*m_icospacing))  + (m_total_launchers*  (m_icodimension+(m_icodimension*m_icospacing))  )) + ((total_taskpreviews*(m_icodimension+(m_icodimension*m_icospacing))) + (m_icospacing)) ); pos_y = rect.height()- (m_icodimension+((m_icodimension/5)*2) );
       m_orientation = QString("horizontal");
     }
     
     else if ( m_position == QString("top_center") )
     {
-      pos_x = ( rect.width()/2)-( (  (((m_icodimension/1.7)*2)-(m_icodimension*m_icospacing))  + (m_total_launchers*  (m_icodimension+(m_icodimension*m_icospacing))  ))/2  ); pos_y = 0;
+      pos_x = ( rect.width()/2)-( (  (((m_icodimension/1.7)*2)-(m_icodimension*m_icospacing))  + (m_total_launchers*  (m_icodimension+(m_icodimension*m_icospacing))  )+((total_taskpreviews*(m_icodimension+(m_icodimension*m_icospacing))) + (m_icospacing)))/2  ); pos_y = 0;
       m_orientation = QString("horizontal");
     }
     else if ( m_position == QString("top_left") )
-    {
-      pos_x = 0; pos_y = 0;m_orientation = QString("horizontal");
-    }
+    {pos_x = 0; pos_y = 0;m_orientation = QString("horizontal");}
     else if ( m_position == QString("top_right") )
     {
-      pos_x = ( rect.width())-( (  (((m_icodimension/1.7)*2)-(m_icodimension*m_icospacing))  + (m_total_launchers*  (m_icodimension+(m_icodimension*m_icospacing))  ))  ); pos_y = 0;m_orientation = QString("horizontal");
+      pos_x = ( rect.width())-( (  (((m_icodimension/1.7)*2)-(m_icodimension*m_icospacing))  + (m_total_launchers*  (m_icodimension+(m_icodimension*m_icospacing))  ))+ ((total_taskpreviews*(m_icodimension+(m_icodimension*m_icospacing))) + (m_icospacing))  ); pos_y = 0;m_orientation = QString("horizontal");
     }
     
     
+    
     //Vert
     else if ( m_position == QString("left_center") )
     {
-      pos_x = 0; pos_y = ( rect.height()/2)-( (  (((m_icodimension/1.7)*2)-(m_icodimension*m_icospacing))  + (m_total_launchers*  (m_icodimension+(m_icodimension*m_icospacing))  ))/2  );
+      pos_x = 0; pos_y = ( rect.height()/2)-( (  (((m_icodimension/1.7)*2)-(m_icodimension*m_icospacing))  + (m_total_launchers*  (m_icodimension+(m_icodimension*m_icospacing))  )+((total_taskpreviews*(m_icodimension+(m_icodimension*m_icospacing))) + (m_icospacing)))/2  );
       m_orientation = QString("vertical");
     }
     else if ( m_position == QString("left_top") )
-    {
-      pos_x = 0; pos_y = 0;m_orientation = QString("vertical");
-    }
+    {pos_x = 0; pos_y = 0;m_orientation = QString("vertical");}
     else if ( m_position == QString("left_buttom") )
     {
-      pos_x = 0; pos_y = ( rect.height() )-( (  (((m_icodimension/1.7)*2)-(m_icodimension*m_icospacing))  + (m_total_launchers*  (m_icodimension+(m_icodimension*m_icospacing))  ))  );
+      pos_x = 0; pos_y = ( rect.height() )-( (  (((m_icodimension/1.7)*2)-(m_icodimension*m_icospacing))  + (m_total_launchers*  (m_icodimension+(m_icodimension*m_icospacing))  )+ ((total_taskpreviews*(m_icodimension+(m_icodimension*m_icospacing))) + (m_icospacing)))  );
       m_orientation = QString("vertical");
     }
     
     else if ( m_position == QString("right_center") )
     {
-      pos_x = ( rect.width())-( (  (m_icodimension/5)  + ( (m_icodimension+(m_icodimension/5))  ))  ); pos_y = ( rect.height()/2)-( (  (((m_icodimension/1.7)*2)-(m_icodimension*m_icospacing))  + (m_total_launchers*  (m_icodimension+(m_icodimension*m_icospacing))  ))/2  );
+      pos_x = ( rect.width())-( (  (m_icodimension/5)  + ( (m_icodimension+(m_icodimension/5))  ))  ); pos_y = ( rect.height()/2)-( (  (((m_icodimension/1.7)*2)-(m_icodimension*m_icospacing))  + (m_total_launchers*  (m_icodimension+(m_icodimension*m_icospacing))  )+((total_taskpreviews*(m_icodimension+(m_icodimension*m_icospacing))) + (m_icospacing)))/2  );
       m_orientation = QString("vertical");
     }
     else if ( m_position == QString("right_top") )
@@ -1144,29 +1284,37 @@ void daisy::resizeRepos( bool m_resetpos )
     }
     else if ( m_position == QString("right_buttom") )
     {
-      pos_x = ( rect.width())-( (  (m_icodimension/5)  + ( (m_icodimension+(m_icodimension/5))  ))  ); pos_y = ( rect.height() )-( (  (((m_icodimension/1.7)*2)-(m_icodimension*m_icospacing))  + (m_total_launchers*  (m_icodimension+(m_icodimension*m_icospacing))  ))  );
+      pos_x = ( rect.width())-( (  (m_icodimension/5)  + ( (m_icodimension+(m_icodimension/5))  ))  ); pos_y = ( rect.height() )-( (  (((m_icodimension/1.7)*2)-(m_icodimension*m_icospacing))  + (m_total_launchers*  (m_icodimension+(m_icodimension*m_icospacing))  )+ ((total_taskpreviews*(m_icodimension+(m_icodimension*m_icospacing))) + (m_icospacing)))  );
       m_orientation = QString("vertical");
     }
     
     
     
     
-    
-    
     setElements();
     
     
-    if ( m_orientation== QString("vertical") )
+    
+    
+    // Resize
+    if ( m_orientation == QString("horizontal") )
     {
-      sz_x = m_icodimension + ((m_icodimension/5)*2);
-      sz_y = (((m_icodimension/1.7)*2) - (m_icodimension*m_icospacing)) + (m_total_launchers*(m_icodimension+(m_icodimension*m_icospacing)));
+      sz_x = (((m_icodimension/1.7)*2) - (m_icodimension*m_icospacing)) + (m_total_launchers*(m_icodimension+(m_icodimension*m_icospacing))) +((total_taskpreviews*(m_icodimension+(m_icodimension*m_icospacing))) + (m_icospacing));
+      sz_y = m_icodimension + ((m_icodimension/5)*2);
     }
     else
     {
-      sz_x = (((m_icodimension/1.7)*2) - (m_icodimension*m_icospacing)) + (m_total_launchers*(m_icodimension+(m_icodimension*m_icospacing)));
-      sz_y = m_icodimension + ((m_icodimension/5)*2);
+      sz_x = m_icodimension + ((m_icodimension/5)*2);
+      sz_y = (((m_icodimension/1.7)*2) - (m_icodimension*m_icospacing)) + (m_total_launchers*(m_icodimension+(m_icodimension*m_icospacing)))+((total_taskpreviews*(m_icodimension+(m_icodimension*m_icospacing))) + (m_icospacing));
     }
+    
     resize( sz_x, sz_y );
+    setMinimumSize(sz_x,sz_y);
+    setMaximumSize(sz_x,sz_y);
+    
+    
+    
+    
     
     // Lazzy math FIXME
     if ( m_position == QString("buttom_center") || m_position == QString("buttom_left") || m_position == QString("buttom_right") )
@@ -1179,10 +1327,8 @@ void daisy::resizeRepos( bool m_resetpos )
     }
     
     
-    
     if (!m_inpanel)
     {
-      
       if (!m_floating)
       {
 	setPos(pos_x,pos_y);
@@ -1190,18 +1336,23 @@ void daisy::resizeRepos( bool m_resetpos )
 	setImmutability(Plasma::UserImmutable);
 	if ( m_autohide )m_tracker_tmr->start();
       }
-      
       else
       {
-	if (m_resetpos)setPos(pos_x,pos_y);
+	//if (m_resetpos)setPos(pos_x,pos_y);
 	setAspectRatioMode(Plasma::FixedSize);
       }
-      
     }
     
     
+    
   }
   
+  
+  
+  
+  
+  
+  
 //   setContainmentType(Containment::CustomPanelContainment);
   emit spaceReserver(false);
 }
@@ -1290,6 +1441,11 @@ QList< QList <double> > daisy::calcWidgetPositions( const QRect &contentsRect )
 	
 	m_indico_posX << ( (m_icodimension/1.7) + ( i* (m_icodimension+(m_icodimension*m_icospacing)) ) )+m_zoomposdiff[i];
 	m_indico_posY << (m_icodimension/5) + (m_ico_szY[i]/5)+m_zoomposdiff[i];
+	
+	
+	last_icopos.clear();
+	last_icopos << ( (m_icodimension/1.7) + ( (i+1)* (m_icodimension+(m_icodimension*m_icospacing)) ) );
+	last_icopos << (m_icodimension/5.5);
       }
       
       else
@@ -1299,6 +1455,11 @@ QList< QList <double> > daisy::calcWidgetPositions( const QRect &contentsRect )
 	
 	m_indico_posY << ( (m_icodimension/1.7) + (i* ((m_icodimension*m_icospacing)+m_icodimension)) )+ (m_ico_szY[i]/5)+m_zoomposdiff[i];
 	m_indico_posX << (m_icodimension/5)+m_zoomposdiff[i];
+	
+	
+	last_icopos.clear();
+	last_icopos << (m_icodimension/5.5);
+	last_icopos << ( (m_icodimension/1.7) + ( (i+1)* (m_icodimension+(m_icodimension*m_icospacing)) ) );
       }
       
     }
@@ -1344,6 +1505,9 @@ void daisy::setTheme()
   else if ( m_theme_t == QString("mach") )
   {m_backframe_t.setImagePath("widgets/dock-background-mach");}
   
+  else if ( m_theme_t == QString("mach-2") )
+  {m_backframe_t.setImagePath("widgets/dock-background-mach-2");}
+  
   else if ( m_theme_t == QString("shiny-black") )
   {m_backframe_t.setImagePath("widgets/dock-background-shiny-black");}
   
@@ -1399,74 +1563,6 @@ void daisy::setElements()
 
 
 
-
-
-
-
-
-
-
-//-----------------------------------------------------------------------------
-// Swap to linear role
-void daisy::swapLinear()
-{
-  m_circpos = scenePos();
-  m_paint_tmr->stop();m_to_paint=int(0);
-  setBusy(false);
-  
-  m_type=QString("standard_dock");
-  QTimer::singleShot(700, this, SLOT(delayResize()));
-  emit metamorphosis( m_type );
-  
-}
-
-
-
-//-----------------------------------------------------------------------------
-// Swap circular role
-void daisy::swapCircular()
-{
-  
-  m_paint_tmr->stop();m_to_paint=int(0);
-  QString m_type_temp = m_type;
-  setBusy(false);
-  
-  if ( m_type == QString("circular_dock") ) m_type=QString("media_controller");
-  else if ( m_type == QString("media_controller") ) m_type=QString("circular_dock");
-  else if ( m_type == QString("standard_dock") )
-  {
-    QTimer::singleShot(700, this, SLOT(delayResize()));
-    m_type=QString("circular_dock");
-  }
-  
-  emit metamorphosis( m_type_temp );
-  
-}
-
-
-
-
-//-----------------------------------------------------------------------------
-// Metamorphosis
-void daisy::metamorphosis( QString m_type_temp )
-{
-  
-  KConfigGroup cg = config();
-  cg.writeEntry("type", m_type);
-  emit configNeedsSaving();
-  refurbishLaunchers();
-  
-  if ( m_type_temp == QString("standard_dock") )
-    resizeRepos( true );
-  
-  m_up_tmr->setInterval(500);m_up_tmr->start();
-  
-}
-
-
-
-
-
 //-----------------------------------------------------------------------------
 // Toggle state
 //FIXME does not work when all the widgets in the plasma workspace are locked. 
@@ -1561,15 +1657,40 @@ void daisy::updateToolTipContent()
 // Clear tooltips
 void daisy::clearToolTipContent()
 {
-  
+  Plasma::ToolTipContent toolTipData;toolTipData.setAutohide(false);
   int all = m_widgets.size();
+  
+  
   for (int i = 0; i < all; ++i)
   {
-    if (m_show_trash)
-    {if ( m_widgets[i] != m_widgets[m_trash_spot] ) Plasma::ToolTipManager::self()->clearContent(m_widgets[i]);}
-    else Plasma::ToolTipManager::self()->clearContent(m_widgets[i]);
+    if (m_show_trash && m_widgets[i] == m_widgets[m_trash_spot])
+    {}
+    
+    else
+    {
+      if (m_showtooltips_l && m_showtooltips_i && m_type != QString("media_controller") )
+      {
+	toolTipData.setMainText( m_values[i][1] );
+	if ( m_tiptype == "thumbnail" || m_tiptype == "icon" )
+	  toolTipData.setImage( KIcon( m_values[i][2]).pixmap(IconSize(KIconLoader::Panel)) );
+	Plasma::ToolTipManager::self()->setContent(m_widgets[i], toolTipData);
+      }
+      
+      else
+	Plasma::ToolTipManager::self()->clearContent(m_widgets[i]);
+    }
   }
   
+  
+  
+  if ( m_taskpreviews && m_type == QString("standard_dock") )
+  {
+    all = m_taskicons.size();
+    for (int i = 0; i < all; ++i)
+    {Plasma::ToolTipManager::self()->clearContent(m_taskicons[i]);}
+  }
+  
+  
 }
 
 
@@ -1612,17 +1733,68 @@ void daisy::mouseTracker(void)
 
 
 
+
 //-----------------------------------------------------------------------------
-// HACK ghost showing on repaint
-void daisy::delayUpdate(void)
+// Swap to linear role
+void daisy::swapLinear()
 {
-  m_up_tmr->stop();
-  if (!m_drawerscleared)clearDrawers();
-  getAllTasks();
-  emit update();
+  m_circpos = scenePos();
+  m_paint_tmr->stop();m_to_paint=int(0);
+  setBusy(false);
+  
+  m_type=QString("standard_dock");
+  QTimer::singleShot(550, this, SLOT(delayResize()));
+  emit metamorphosis( m_type );
+  
 }
 
 
+
+//-----------------------------------------------------------------------------
+// Swap circular role
+void daisy::swapCircular()
+{
+  
+  m_paint_tmr->stop();m_to_paint=int(0);
+  QString m_type_temp = m_type;
+  setBusy(false);
+  
+  if ( m_type == QString("circular_dock") ) m_type=QString("media_controller");
+  else if ( m_type == QString("media_controller") ) m_type=QString("circular_dock");
+  else if ( m_type == QString("standard_dock") )
+  {
+    QTimer::singleShot(550, this, SLOT(delayResize()));
+    m_type=QString("circular_dock");
+  }
+  
+  emit metamorphosis( m_type_temp );
+  
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// Metamorphosis
+void daisy::metamorphosis( QString m_type_temp )
+{
+  
+  KConfigGroup cg = config();
+  cg.writeEntry("type", m_type);
+  emit configNeedsSaving();
+  refurbishLaunchers();
+  
+  if ( m_type_temp == QString("standard_dock") )
+  {
+    resizeRepos(true);
+  }
+  m_up_tmr->setInterval(100);m_up_tmr->start();
+  
+}
+
+
+
+
 //-----------------------------------------------------------------------------
 // HACK delay resize
 void daisy::delayResize(void)
@@ -1631,22 +1803,72 @@ void daisy::delayResize(void)
   m_up_tmr->stop();m_rz_tmr->stop();
   KConfigGroup cg = config();
   cg.writeEntry("icodimension", m_icodimension);
-  resizeRepos( true );
+  resizeRepos(true);
   emit update();
   m_up_tmr->setInterval(500);m_up_tmr->start();
   
 }
 
 
+//-----------------------------------------------------------------------------
+// HACK ghost showing on repaint
+void daisy::delayUpdate(void)
+{
+  m_up_tmr->stop();
+  if (!m_drawerscleared)clearDrawers();
+  getAllTasks();
+  setVisible(true);
+  emit update();
+}
+
+
 
 //-----------------------------------------------------------------------------
-//HACK NEEDS TO BE RESIZED TWICE IN KDE 4.3 ?  FIXME
-void daisy::delay_Resize(void)
+// HACK Delayed Resize circular
+void daisy::resetResize(void)
 {
-  
-  m_rp_tmr->stop();
-  resizeRepos( true );
-  
+  resize(330,330);
+  setPos(QPointF(62,268));
+  setMaximumSize(2330,2330);
+}
+
+
+
+//-----------------------------------------------------------------------------
+// HACK Avoid repeating execution
+void daisy::delayExec(void)
+{
+  m_dlx_tmr->stop();
+  KRun::runCommand( m_values[m_dexval][0], m_values[m_dexval][1], m_values[m_dexval][2], NULL, NULL );
+}
+
+
+
+//-----------------------------------------------------------------------------
+// HACK Avoid repeating shade
+void daisy::delayShade(void)
+{
+  m_dlt_tmr->stop();
+  toggShadeTask( m_workwid );
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// Close a task
+void daisy::taskClosing(void)
+{
+  m_workwid = 0;
+  for (int i = 0; i < total_taskpreviews; ++i)
+  {
+    if (m_taskicons[i]->isUnderMouse())
+    {
+      m_workwid = showedTasks[i];
+      break;
+    }
+  }
+  closeTask(m_workwid);
 }
 
 
